/*--------------------------Gate_Delay_090.c  -----------------------------------
 * 
 * 
 * Software for Gate Delay
 * 
 * 
 * Hardware Version:  PCB Pulses 0.9.5-B 2020_05 and Pulses 1.0.1-F 2020_06
 * 
 * 
 * Device check default values:
 * avrdude -p m328p -c avrisp2                   						; command
 * avrdude: Device signature = 0x1e950f   (probably m328p)	; default values for m328p
 * avrdude: safemode: Fuses OK (E:FF, H:D9, L:62) 					; default values for m328p
 * 
 * 
 * Fuse settings for external quarz (20MHz)
 * avrdude -p m328p -c avrisp2 -U lfuse:w:0xe7:m 
 * avrdude -p m328p -c avrisp2 -U hfuse:w:0xd9:m 
 * 
 * 
 * compile:
 * avr-gcc -Wall -Os -mmcu=atmega328p Gate_Delay_090.c -o Gate_Delay_090.elf
 * avr-objcopy -O ihex -j .text -j .data Gate_Delay_090.elf Gate_Delay_090.hex
 * avrdude -p m328p -c avrisp2 -U flash:w:Gate_Delay_090.hex
 * 
 * 
 * ADC0: Coarse potentiometer
 * ADC1: Fine potentiometer
 * INT0: Rising edge
 * INT1: Falling edge
 * PD5: Trim switch input
 * PD6: Gate output
 * 
 * 
 * ---------------------------------------------------------------------------------------
 *
 * Start 2021_05 Harald
 * 
 */

#define F_CPU 20000000UL	//CPU speed used in delay.h
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
/* ------------ Variables --------------------------*/

volatile uint8_t coarse_delay;			// Holds result from ADC0 "coarse" potentiometer
volatile uint8_t fine_delay;			// Holds result from ADC1 "fine" potentiometer

/* ---------------------------- functions ------------------------------*/

/* ADC initialisieren */
void ADC_Init(void) {
 
  uint16_t result;
 
  ADMUX |= (1 << REFS0);					// reference voltage on AVCC    
	ADMUX |= (1 << ADLAR);					// left adjust result, return only 8bits
  // Bit ADFR ("free running") in ADCSRA steht beim Einschalten
  // schon auf 0, also single conversion
  ADCSRA |= (1<<ADPS2);    				// Frequenzvorteiler 8MHz / 16 = 250kHz empf: 50 - 200 kHz
  ADCSRA |= (1<<ADEN);            // ADC activate
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) ) {         // auf Abschluss der Konvertierung warten
  }
  /* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nächsten
     Wandlung nicht übernommen. */
  result = ADCW;
}


/* ADC Einzelmessung */
uint8_t ADC_Read( uint8_t channel )
{
  // Kanal waehlen, ohne andere Bits zu beeinflußen
  ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
  ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
  while (ADCSRA & (1<<ADSC) ) 
	{   // auf Abschluss der Konvertierung warten
  }
  
  return (ADCH);           		    // ADC High byte auslesen und zurückgeben
}                             

void init_Ports(void)
 {
  DDRD &= ~(1 << PD2);		    // Interrupt0 PD2 and PD3 as input
  DDRD &= ~(1 << PD3);        
	DDRD &= ~(1 << PD5);				// Trim switch input
  DDRD |= (1 << PD6);					// Gate Output 
	PORTD |= (1<<PD5);					// PD5 activat pullup resistor
 }
	
void set_interrupt(void)
 {
	EIMSK |= (1<<INT0);							// enable INT0
	EIMSK |= (1<<INT1);			        // enable INT1
  EICRA |= (1<<ISC00)|(1<<ISC01); // INT0 - Rising edge on INT0 generates an interrupt request
  EICRA |= (1<<ISC11);            // INT1 - Falling edge of INT1 generates interrupt request  
 }

volatile uint8_t i;

//INT0 interrupt 
ISR(INT0_vect) 										// Gate input changes to high
{ 
	for (i=0;i<fine_delay;i++)			// "fine" pot delay
		_delay_us(100);
	for (i=0;i<coarse_delay;i++)		// "coarse" pot delay
		_delay_us(1000);
	PORTD |= (1 << PD6);						// Gate out high
} 

//INT1 interrupt 
ISR(INT1_vect) 										// Gate input changes to low
{ 
	if (bit_is_set(PIND,PD5))
	{
	for (i=0;i<fine_delay;i++)			// "fine" pot delay
		_delay_us(100);
	for (i=0;i<coarse_delay;i++)		// "coarse" pot delay
		_delay_us(1000);
	}
	PORTD &= ~(1 << PD6);						// Gate out Low
} 


/* ----------------------MAIN ---------------------------*/


int main (void)
{	
	cli();
	init_Ports();
	set_interrupt();
	ADC_Init();
	sei();																// Interrupts activate
    while (1)														// For ever
		{
			coarse_delay = ADC_Read(0);				// "Coarse" pot read
			fine_delay = ADC_Read(1);					// "Fine" pot read
		}
}
